
/**
 * 3d secure authentication using Pci Bookings Orchestra service
 */
var ts3DSecure = {

    webSocket: null,
    TDSAST: null,           // The ThreeDSecure Session Token, PCIB generated token to identify the WebSocket
    fingerPrintTimer: null, // When receiving FingerPrint message from the server a following message is expected in the following 10 seconds, in case a message isn't received we send a request 
    // to the server for the finger print status (retrieving the message)

/**
 * Main function to perform a 3d secure authentication session
 */
    perform3ds: function() {
        return "Hello World";
    },

    // init3DSecAuthSession
    init3DSecAuthSession: function (cardData, authTokenQueryString) {

        var options = {
            url: cardCapture.endpointUrl + "/api/3DSecure?" + authTokenQueryString,
            type: 'POST',
            crossDomain: true,
            cache: false,
            xhrFields: {
                withCredentials: true
            },
            contentType: "application/json",
            data: JSON.stringify(cardData),
            async: true,
            dataType: "json",
            success: function (data, textStatus, xhr) {
                var locationVal = xhr.getResponseHeader("Location");
                ts3DSecure.TDSAST = locationVal.substring(locationVal.lastIndexOf('/') + 1);

                ts3DSecure.handleServerMessage(JSON.parse(xhr.responseText));
            },
            error: function (jqXHR, textStatus, errorThrown) {
                cardCapture.toggleThrobber(false);
                cardCapture.handleInit3DSecSessionFailure(jqXHR);
            }
        };

        $.ajax(options);
    },

    // openWebSocket
    openWebSocket: function (onOpenCallback) {
        try {
            // find deployment stage (including leading '-')
            var domainPrefixLength = location.hostname.indexOf('.');
            var domainPrefix = location.hostname.substr(0, domainPrefixLength);
            var n = domainPrefix.lastIndexOf("-");
            var stage = (n === -1) ? ""  /* production */ : domainPrefix.substring(n) /* other stage */;

            // construct Websocket Url, e.g. wss://3dws-plan.pcibooking.net/ or wss://3dws.pcibooking.net/
            var webSocketURL = 'wss://3dws' + stage + '.pcibooking.net?tdsast=' + this.TDSAST;

            // Initiate a websocket session and pass the TDSAST as 'protocol'. 
            // ways to add headers are not available.Pssing TDSAST in the path is not supported by AWS ApiGateway
            this.webSocket = new WebSocket(webSocketURL);
        }
        catch (error) {
            console.error(error);
            return;
        }

        this.bindWebSocketEvents(onOpenCallback);
    },

    // closeWebSocket
    closeWebSocket: function () {
        // WebSocket readyStates: 0: CONNECTING, 1: OPEN, 2: CLOSING, 3: CLOSED
        console.log("Disconnect Websocket was triggered; ignoring (work around) and logging trace");
        console.trace();
        return;
    //    if (this.webSocket && (ts3DSecure.webSocket.readyState === 0 || ts3DSecure.webSocket.readyState === 1))
    //        this.webSocket.close();
    },

    // bindWebSocketEvents
    bindWebSocketEvents: function (onOpenCallback) {
        // OnOpen
        this.webSocket.onopen = onOpenCallback;

        // OnError
        this.webSocket.onerror = function (event) {
            console.log(event.data);
        };

        // OnMessage
        this.webSocket.onmessage = this.onMessage;

        //// On window close
        //$(window).on('beforeunload', function () {
        //    console.log("beforeunload was triggered; A WbSocket disconnect will be sent");
        //    ts3DSecure.closeWebSocket();
        //});
    },

    // onMessage: handles a message recieved from the websocket
    onMessage: function (event) {
        ts3DSecure.handleServerMessage(JSON.parse(event.data));
    },

    // handleServerMessage
    handleServerMessage: function (message) {
        ts3DSecure.clearFingerPrintTimer();
        cardCapture.clear3DSecProcessTimer();
        cardCapture.toggleThrobber(false);

        console.log('3DS: Recieved ' + message.MessageType + ' message from server');
        switch (message.MessageType.toLowerCase()) {
            case 'authenticated':
                cardCapture.goToPayment(ts3DSecure.TDSAST, 'Authenticated');
                break;
            case 'cardnotenrolled':
                cardCapture.goToPayment(ts3DSecure.TDSAST, 'CardNotEnrolled');
                break;
            case 'challengerequired':
                ts3DSecure.openWebSocket(function () {
                    $('#3DS-Hidden').attr('src', '');   // Clear hidden iframe
                    // set postmessage on challenge iFrame load
                    $('#3DS').on("load", function () {
                        console.log("3DS challenge iframe loaded");
                        frameNotifications.sendThreeDsChallengeLoadedMessage();
                    });
                    $('#3DS').attr('src', message.ascUri).css('display', 'inline');
                    $('html, body').animate({ scrollTop: $("#3DS").offset().top }, 500);
                });
                break;
            case 'fingerprintrequired':
                cardCapture.toggleThrobber(true);
                ts3DSecure.openWebSocket(function () {
                    // create an iframe 
                    console.log("Creating an iFrame");
                    $('<iframe id="threeDsHiddenIframe" name="threeDsHiddenIframe" style="display: none;">').appendTo('body');

                    // add form
                    console.log("Adding form to iFrame");
                    var form =
                        "<form method='post' id='frmRedirect' action='" + message.FingerPrintUri + "'><input type='hidden' name='threeDSMethodData' value='" + message.FingerPrintData + "' /><input type='submit'/></form>";
                    $('#threeDsHiddenIframe').contents().find('body').html(form);

                    // add script to submit the form
                    console.log("add script to submit the form");
                    var scriptTag = "<script>document.getElementById('frmRedirect').submit();<\/script>";
                    $('#threeDsHiddenIframe').contents().find("body").append(scriptTag);

                    // Starting timers
                    console.log("Starting timers");
                    cardCapture.start3DSecProcessTimer();
                    ts3DSecure.startFingerPrintTimer();
                });
                break;
            case 'technicalproblem':
                cardCapture.handle3DSTechProblem(ts3DSecure.TDSAST, message.Reason + "[" + message.Reference + "]");
                break;
            case 'rejected':
                cardCapture.handle3DSReject(message.Reason);
                break;
            default:
                console.error('Received unknown message - ' + message.MessageType);
        }
    },

    startFingerPrintTimer: function () {
        console.log('Starting FingerPrint timer');
        ts3DSecure.fingerPrintTimer = setTimeout(function () {
            console.log('FingerPrintStatus timer was triggered; Trying to obtain FigerPrint status');
            // This ajax call will trigger the retrieval of the finger print status, it will be received through the websocket
            var options = {
                url: cardCapture.endpointUrl + "/api/3DSecure/" + ts3DSecure.TDSAST + "/fingerPrintStatus",
                type: 'GET',
                crossDomain: true,
                cache: false,
                xhrFields: {
                    withCredentials: true
                },
                async: true,
                success: function (data, textStatus, xhr) {
                    // No further action needed since the finger print status will be received through the socket
                    console.log('FingerPrintStatus timer was triggered successfuly');
                },
                error: function (jqXHR, textStatus, errorThrown) {
                    console.log('FingerPrintStatus timer was triggered with error');
                    cardCapture.toggleThrobber(false);
                    cardCapture.handle3DSTechProblem(ts3DSecure.TDSAST, errorThrown);
                }
            };

            $.ajax(options);
        }, 20000);
    },

    // clearFingerPrintTimer
    clearFingerPrintTimer: function () {
        console.log('Clearing FingerPrint timer');
        clearTimeout(ts3DSecure.fingerPrintTimer);
    }
};